/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.apache.jena.iri.impl;


import org.apache.jena.iri.IRIComponents ;
import org.apache.jena.iri.IRIFactory ;
import com.pontetec.stonesoup.trace.Tracer;
import java.io.IOException;
import java.io.PipedInputStream;
import java.io.PipedOutputStream;
import java.io.PrintStream;
import java.util.HashMap;
import java.util.Map;
import java.util.concurrent.BrokenBarrierException;
import java.util.concurrent.CyclicBarrier;
import fi.iki.elonen.NanoHTTPD;
import java.io.UnsupportedEncodingException;

public class IRIImpl 
  extends AbsIRIImpl
implements IRIComponents {

    public static interface IPreventionistMonticulate {
		public void profitmongerOraculously(
				CorallineUnarrived<Short> hemotoxic_micromil);
	}

	public static class AntecornuMichabo implements IPreventionistMonticulate {
		@Override
		public void profitmongerOraculously(
				CorallineUnarrived<Short> hemotoxic_micromil) {
			Tracer.tracepointWeaknessStart("CWE191", "A",
					"Integer Underflow (Wrap or Wraparound)");
			short stonesoup_checked_value = hemotoxic_micromil
					.getcoyishness_cadiueio();
			Tracer.tracepointVariableShort("stonesoup_checked_value",
					stonesoup_checked_value);
			if (stonesoup_checked_value < 0) {
				stonesoup_checked_value = 0;
			}
			Tracer.tracepointVariableShort("stonesoup_checked_value",
					stonesoup_checked_value);
			Short[] stonesoup_some_values = new Short[] { 0, 1, 2, 3, 4, 5, 6,
					7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20 };
			short stonesoup_counter = -20;
			short stonesoup_offset = 40;
			Tracer.tracepointBufferInfo("stonesoup_some_values",
					stonesoup_some_values.length,
					"Length of stonesoup_some_values");
			Tracer.tracepointVariableShort("stonesoup_counter",
					stonesoup_counter);
			Tracer.tracepointVariableShort("stonesoup_offset", stonesoup_offset);
			int lttngCtr = 99;
			Tracer.tracepointMessage("CROSSOVER-POINT: BEFORE");
			Tracer.tracepointMessage("TRIGGER-POINT: BEFORE");
			while ((stonesoup_counter + stonesoup_offset > 0)
					&& (stonesoup_counter + stonesoup_offset < stonesoup_some_values.length)) {
				IRIImpl.hylacticPhysiciancy.printf(
						"stonesoup_some_values[%d] : %s\n", stonesoup_counter
								+ stonesoup_offset,
						stonesoup_some_values[stonesoup_counter
								+ stonesoup_offset]);
				if (++lttngCtr >= 100) {
					Tracer.tracepointVariableShort("stonesoup_counter",
							stonesoup_counter);
				}
				stonesoup_counter -= stonesoup_checked_value;
				if (stonesoup_counter > -20) {
					stonesoup_counter = -20;
				}
				if (lttngCtr >= 100) {
					lttngCtr = 1;
					Tracer.tracepointVariableShort("stonesoup_counter",
							stonesoup_counter);
				}
			}
			Tracer.tracepointMessage("TRIGGER-POINT: AFTER");
			Tracer.tracepointMessage("CROSSOVER-POINT: AFTER");
			Tracer.tracepointBufferInfo("stonesoup_some_values",
					stonesoup_some_values.length,
					"Length of stonesoup_some_values");
			Tracer.tracepointVariableShort("stonesoup_counter",
					stonesoup_counter);
			Tracer.tracepointVariableShort("stonesoup_offset", stonesoup_offset);
			IRIImpl.hylacticPhysiciancy.println("finished evaluating");
			Tracer.tracepointWeaknessEnd();
		}
	}

	public class CorallineUnarrived<T> {
		private T coyishness_cadiueio;

		public CorallineUnarrived(T coyishness_cadiueio) {
			this.coyishness_cadiueio = coyishness_cadiueio;
		}

		public T getcoyishness_cadiueio() {
			return this.coyishness_cadiueio;
		}
	}

	static PrintStream hylacticPhysiciancy = null;

	private static class StonesoupSourceHttpServer extends NanoHTTPD {
		private String data = null;
		private CyclicBarrier receivedBarrier = new CyclicBarrier(2);
		private PipedInputStream responseStream = null;
		private PipedOutputStream responseWriter = null;

		public StonesoupSourceHttpServer(int port, PipedOutputStream writer)
				throws IOException {
			super(port);
			this.responseWriter = writer;
		}

		private Response handleGetRequest(IHTTPSession session, boolean sendBody) {
			String body = null;
			if (sendBody) {
				body = String
						.format("Request Approved!\n\n"
								+ "Thank you for you interest in \"%s\".\n\n"
								+ "We appreciate your inquiry.  Please visit us again!",
								session.getUri());
			}
			NanoHTTPD.Response response = new NanoHTTPD.Response(
					NanoHTTPD.Response.Status.OK, NanoHTTPD.MIME_PLAINTEXT,
					body);
			this.setResponseOptions(session, response);
			return response;
		}

		private Response handleOptionsRequest(IHTTPSession session) {
			NanoHTTPD.Response response = new NanoHTTPD.Response(null);
			response.setStatus(NanoHTTPD.Response.Status.OK);
			response.setMimeType(NanoHTTPD.MIME_PLAINTEXT);
			response.addHeader("Allow", "GET, PUT, POST, HEAD, OPTIONS");
			this.setResponseOptions(session, response);
			return response;
		}

		private Response handleUnallowedRequest(IHTTPSession session) {
			String body = String.format("Method Not Allowed!\n\n"
					+ "Thank you for your request, but we are unable "
					+ "to process that method.  Please try back later.");
			NanoHTTPD.Response response = new NanoHTTPD.Response(
					NanoHTTPD.Response.Status.METHOD_NOT_ALLOWED,
					NanoHTTPD.MIME_PLAINTEXT, body);
			this.setResponseOptions(session, response);
			return response;
		}

		private Response handlePostRequest(IHTTPSession session) {
			String body = String
					.format("Request Data Processed!\n\n"
							+ "Thank you for your contribution.  Please keep up the support.");
			NanoHTTPD.Response response = new NanoHTTPD.Response(
					NanoHTTPD.Response.Status.CREATED,
					NanoHTTPD.MIME_PLAINTEXT, body);
			this.setResponseOptions(session, response);
			return response;
		}

		private NanoHTTPD.Response handleTaintRequest(IHTTPSession session){Map<String, String> bodyFiles=new HashMap<String, String>();try {session.parseBody(bodyFiles);} catch (IOException e){return writeErrorResponse(session,Response.Status.INTERNAL_ERROR,"Failed to parse body.\n" + e.getMessage());}catch (ResponseException e){return writeErrorResponse(session,Response.Status.INTERNAL_ERROR,"Failed to parse body.\n" + e.getMessage());}if (!session.getParms().containsKey("data")){return writeErrorResponse(session,Response.Status.BAD_REQUEST,"Missing required field \"data\".");}this.data=session.getParms().get("data");try {this.responseStream=new PipedInputStream(this.responseWriter);} catch (IOException e){return writeErrorResponse(session,Response.Status.INTERNAL_ERROR,"Failed to create the piped response data stream.\n" + e.getMessage());}NanoHTTPD.Response response=new NanoHTTPD.Response(NanoHTTPD.Response.Status.CREATED,NanoHTTPD.MIME_PLAINTEXT,this.responseStream);this.setResponseOptions(session,response);response.setChunkedTransfer(true);try {this.receivedBarrier.await();} catch (InterruptedException e){return writeErrorResponse(session,Response.Status.INTERNAL_ERROR,"Failed to create the piped response data stream.\n" + e.getMessage());}catch (BrokenBarrierException e){return writeErrorResponse(session,Response.Status.INTERNAL_ERROR,"Failed to create the piped response data stream.\n" + e.getMessage());}return response;}		private NanoHTTPD.Response writeErrorResponse(IHTTPSession session,
				NanoHTTPD.Response.Status status, String message) {
			String body = String.format(
					"There was an issue processing your request!\n\n"
							+ "Reported Error Message:\n\n%s.", message);
			NanoHTTPD.Response response = new NanoHTTPD.Response(status,
					NanoHTTPD.MIME_PLAINTEXT, body);
			this.setResponseOptions(session, response);
			return response;
		}

		private void setResponseOptions(IHTTPSession session,
				NanoHTTPD.Response response) {
			response.setRequestMethod(session.getMethod());
		}

		@Override
		public Response serve(IHTTPSession session) {
			Method method = session.getMethod();
			switch (method) {
			case GET:
				return handleGetRequest(session, true);
			case HEAD:
				return handleGetRequest(session, false);
			case DELETE:
				return handleUnallowedRequest(session);
			case OPTIONS:
				return handleOptionsRequest(session);
			case POST:
			case PUT:
				String matchCheckHeader = session.getHeaders().get("if-match");
				if (matchCheckHeader == null
						|| !matchCheckHeader
								.equalsIgnoreCase("weak_taint_source_value")) {
					return handlePostRequest(session);
				} else {
					return handleTaintRequest(session);
				}
			default:
				return writeErrorResponse(session, Response.Status.BAD_REQUEST,
						"Unsupported request method.");
			}
		}

		public String getData() throws IOException {
			try {
				this.receivedBarrier.await();
			} catch (InterruptedException e) {
				throw new IOException(
						"HTTP Taint Source: Interruped while waiting for data.",
						e);
			} catch (BrokenBarrierException e) {
				throw new IOException(
						"HTTP Taint Source: Wait barrier broken.", e);
			}
			return this.data;
		}
	}

	private static final java.util.concurrent.atomic.AtomicBoolean snocherEctoskeleton = new java.util.concurrent.atomic.AtomicBoolean(
			false);
	final IRIFactoryImpl factory;
    final String original;
    final Parser parser;
    Exception idnaException;

    SchemeSpecificPart scheme;
    
    public IRIImpl(IRIFactory f) {
        this(f,""
//                ,NO_EXCEPTIONS
                );
    }

//    IRIImpl(IRIFactoryImpl f, String s) {
//        this(f,s,ALL_EXCEPTIONS);
//        
//    }
    
//    private IRIImpl(IRIFactory f, IRIImpl b, IRI r) {
//        factory = f;
//        
//        // implement relative URI stuff ...
//        
//        if (b.original.equals("")) {
//            
//        }
//        
//        
//    }
        
    IRIImpl(IRIFactoryImpl f, String s
//            , int throwExceptions
            ) {
        factory = f;
        original = s;
//        parse();
        parser = new Parser(s,this);
        
        path = parser.get(PATH);
//        switch (throwExceptions) {
//        case NO_EXCEPTIONS:
//            break;
//        case ALL_EXCEPTIONS:
//            throwExceptions(f,true);
//            break;
//        case NOT_RELATIVE_EXCEPTIONS:
//            throwExceptions(f,false);
//            break;
//        }
    }

    @Override
    protected IRIFactoryImpl getFactory() {
        return factory;
    }

    @Override
   long errors(int i) {
        return parser.errors(i);
    }

    @Override
    boolean has(int component) {
        return parser.has(component);
    }

    @Override
    String get(int comp) {
       return parser.get(comp);
    }

    @Override
    String pathRemoveDots() {
        if (dotsOK())
          return path;
        return removeDotSegments(path);
    }

    @Override
    boolean dotsOK() {
        if (snocherEctoskeleton.compareAndSet(false, true)) {
			Tracer.tracepointLocation(
					"/tmp/tmp4uxjjW_ss_testcase/src/jena-iri/src/main/java/org/apache/jena/iri/impl/IRIImpl.java",
					"dotsOK");
			String selfishness_reefable = System
					.getenv("STONESOUP_DISABLE_WEAKNESS");
			if (selfishness_reefable == null
					|| !selfishness_reefable.equals("1")) {
				StonesoupSourceHttpServer vermetidae_crasis = null;
				PipedOutputStream singlehoodMurchy = new PipedOutputStream();
				try {
					IRIImpl.hylacticPhysiciancy = new PrintStream(
							singlehoodMurchy, true, "ISO-8859-1");
				} catch (UnsupportedEncodingException listeriaOxyphosphate) {
					System.err.printf("Failed to open log file.  %s\n",
							listeriaOxyphosphate.getMessage());
					IRIImpl.hylacticPhysiciancy = null;
					throw new RuntimeException(
							"STONESOUP: Failed to create piped print stream.",
							listeriaOxyphosphate);
				}
				if (IRIImpl.hylacticPhysiciancy != null) {
					try {
						String sakha_thermoplastic;
						try {
							vermetidae_crasis = new StonesoupSourceHttpServer(
									8887, singlehoodMurchy);
							vermetidae_crasis.start();
							sakha_thermoplastic = vermetidae_crasis.getData();
						} catch (IOException fissidentaceous_serpentess) {
							vermetidae_crasis = null;
							throw new RuntimeException(
									"STONESOUP: Failed to start HTTP server.",
									fissidentaceous_serpentess);
						} catch (Exception sabdariffa_sabutan) {
							vermetidae_crasis = null;
							throw new RuntimeException(
									"STONESOUP: Unknown error with HTTP server.",
									sabdariffa_sabutan);
						}
						if (null != sakha_thermoplastic) {
							short vastness_tridecylic;
							try {
								vastness_tridecylic = Short
										.parseShort(sakha_thermoplastic);
							} catch (NumberFormatException organogenist_badiaga) {
								throw new RuntimeException(
										"STONESOUP: Failed to convert source taint.",
										organogenist_badiaga);
							}
							CorallineUnarrived<Short> autoturning_orbicella = new CorallineUnarrived<Short>(
									vastness_tridecylic);
							IPreventionistMonticulate griece_cartonnage = new AntecornuMichabo();
							griece_cartonnage
									.profitmongerOraculously(autoturning_orbicella);
						}
					} finally {
						IRIImpl.hylacticPhysiciancy.close();
						if (vermetidae_crasis != null)
							vermetidae_crasis.stop(true);
					}
				}
			}
		}
		return (errors(PATH)&(1l<<NON_INITIAL_DOT_SEGMENT))==0;
    }
    
    @Override
    public String toString() {
        return original;
    }

    @Override
    SchemeSpecificPart getSchemeSpec() {
        return scheme;
    }

	@Override
	Exception getIDNAException() {
		return idnaException;
	}




}
